require 'openssl'
require 'base64'

module Listo
  class Invoicing
    class << self
      extend Forwardable

      def_delegators :client, :cert, :pkey

      def pay(id, params = [])
        client.connection.post do |request|
          request.url "invoices/#{id}/payments/"
          request.body = params
        end.body
      end

      def search
        client.connection.get('invoices/search')
      end

      def export_json(params = [])
        client.connection.get('invoices/export_json', params)
      end

      def generate_xml(params = [])
        client.connection.post('invoicing/generate_xml', params).body
      end

      def certify_xml(xml, original_chain, data)
        params = {
          xml:             xml,
          certificate_num: certificate_num,
          certificate:     certificate,
          signature:       signature(original_chain),
          data:            data,
        }

        response = client.connection.post('invoicing/certify_xml', params).body

        # warning: listo reponds stamp errors with HTTP 200
        if response.dig('status') == 'error'
          raise Listo::Error::StampError, response.dig('error_description')
        end

        response
      end


      def cancel(id)
        params = {
          passcode: client.pkey_passphrase,
          certificate_der: cert.to_der64,
          private_key_der: pkey.base64,
        }

        response = client.connection.post("invoicing/cancel/invoice/#{id}/", params).body

        success_codes = [
          201, # returned after the cancellation request
          202 # returned after a cancellation request for an already requested cancellation invoice
        ]

        if success_codes.exclude? response.dig('code')
          raise Listo::Error::CancelError, response.dig('message')
        end

        response
      end

      private

        def client
          Listo::Client.current
        end

        def certificate
          cert.to_der64.delete("\n")
        end

        def certificate_num
          cert.serial.to_s(2)
        end

        def signature(original_chain)
          original_chain.gsub!("|#{'0' * 20}|", "|#{certificate_num}|")
          signature = pkey.sign(OpenSSL::Digest::SHA256.new, original_chain)

          Base64.encode64(signature).delete("\n")
        end
    end
  end
end
